@@ -1,7 +1,7 @@
 /* mpfr_cmp_d -- compare a floating-point number with a long double
 
-Copyright 2004, 2006-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2004, 2006-2017 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -27,12 +27,19 @@ mpfr_cmp_ld (mpfr_srcptr b, long double d)
 {
   mpfr_t tmp;
   int res;
+  MPFR_SAVE_EXPO_DECL (expo);
+
+  MPFR_SAVE_EXPO_MARK (expo);
 
   mpfr_init2 (tmp, MPFR_LDBL_MANT_DIG);
   res = mpfr_set_ld (tmp, d, MPFR_RNDN);
   MPFR_ASSERTD (res == 0);
-  res = mpfr_cmp (b, tmp);
-  mpfr_clear (tmp);
 
+  mpfr_clear_flags ();
+  res = mpfr_cmp (b, tmp);
+  MPFR_SAVE_EXPO_UPDATE_FLAGS (expo, __gmpfr_flags);
+
+  mpfr_clear (tmp);
+  MPFR_SAVE_EXPO_FREE (expo);
   return res;
 }
