@@ -1,13 +1,13 @@
-/* Test file for the various mpfr_random fonctions.
+/* Test file for mpfr_urandomb
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 1999-2004, 2006-2017 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
 The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MPFR Library is distributed in the hope that it will be useful, but
@@ -16,9 +16,9 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,160 +26,12 @@ MA 02110-1301, USA. */
 #include "mpfr-test.h"
 
 static void
-test_random (long nbtests, mp_prec_t prec, int verbose)
-{
-  mpfr_t x;
-  int *tab, size_tab, k;
-  double d, av = 0, var = 0, chi2 = 0, th;
-
-  size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
-  tab = (int *) calloc (size_tab, sizeof(int));
-  if (tab == NULL)
-    {
-      fprintf (stderr, "trandom: can't allocate memory in test_random\n");
-      exit (1);
-    }
-
-  mpfr_init2(x, prec);
-
-  for (k = 0; k < nbtests; k++)
-    {
-      mpfr_urandomb (x, RANDS);
-      d = mpfr_get_d1 (x); av += d; var += d*d;
-      tab[(int)(size_tab * d)]++;
-    }
-
-  mpfr_clear(x);
-  if (!verbose)
-    {
-      free(tab);
-      return;
-    }
-
-  av /= nbtests;
-  var = (var / nbtests) - av*av;
-
-  th = (double) nbtests / size_tab;
-
-  printf("Average = %.5f\nVariance = %.5f\n", av, var);
-  printf("Repartition for random. Each integer should be close to %d.\n",
-         (int)th);
-
-  for (k = 0; k < size_tab; k++)
-    {
-      chi2 += (tab[k] - th) * (tab[k] - th) / th;
-      printf("%d ", tab[k]);
-      if (((k+1) & 7) == 0)
-        printf("\n");
-    }
-
-  printf("\nChi2 statistics value (with %d degrees of freedom) : %.5f\n\n",
-         size_tab - 1, chi2);
-
-  printf("\n");
-
-  free(tab);
-  return;
-}
-
-static void
-test_random2 (long nbtests, mp_prec_t prec, int verbose)
+test_urandomb (long nbtests, mpfr_prec_t prec, int verbose)
 {
   mpfr_t x;
   int *tab, size_tab, k, sh, xn;
   double d, av = 0, var = 0, chi2 = 0, th;
-
-  size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
-  tab = (int *) calloc (size_tab, sizeof(int));
-  if (tab == NULL)
-    {
-      fprintf (stderr, "trandom: can't allocate memory in test_random2\n");
-      exit (1);
-    }
-
-  mpfr_init2 (x, prec);
-  xn = 1 + (prec - 1) / mp_bits_per_limb;
-  sh = xn * mp_bits_per_limb - prec;
-
-  for (k = 0; k < nbtests; k++)
-    {
-      mpfr_random2 (x, xn, 0);
-      /* check that lower bits are zero */
-      if (MPFR_MANT(x)[0] & MPFR_LIMB_MASK(sh))
-        {
-          printf ("Error: mpfr_random2() returns invalid numbers:\n");
-          mpfr_print_binary (x); puts ("");
-          exit (1);
-        }
-
-      /* check that the number is normalized */
-      if (! (MPFR_MANT(x)[MPFR_LIMB_SIZE(x) - 1] >> (BITS_PER_MP_LIMB - 1)))
-        {
-          printf ("Error: mpfr_random2() returns unnormalized numbers:\n");
-          mpfr_print_binary (x); puts ("");
-          exit (1);
-        }
-
-      /* check that exponent is in correct range */
-      if (mpfr_get_exp (x) != 0)
-        {
-          printf ("Error: mpfr_random2 (.., .., 0) does not return"
-                  " a 0 exponent:\n");
-          mpfr_print_binary (x); puts ("");
-          exit (1);
-        }
-      d = mpfr_get_d1 (x); av += d; var += d*d;
-      if (d < 1)
-        tab[(int)(size_tab * d)]++;
-    }
-
-  /* test size=0 */
-  mpfr_random2 (x, 0, 0);
-  MPFR_ASSERTN (mpfr_cmp_ui (x, 0) == 0 && MPFR_IS_POS (x));
-  mpfr_set_si (x, -1, GMP_RNDN); /* x is negative */
-  mpfr_random2 (x, 0, 0);
-  MPFR_ASSERTN (mpfr_cmp_ui (x, 0) == 0 && MPFR_IS_POS (x));
-
-  /* test size < 0 */
-  mpfr_random2 (x, -1, 0);
-  MPFR_ASSERTN (MPFR_IS_NEG (x) && MPFR_EXP (x) == 0);
-
-  mpfr_clear (x);
-  if (!verbose)
-    {
-      free(tab);
-      return;
-    }
-
-  av /= nbtests;
-  var = (var / nbtests) - av*av;
-
-  th = (double)nbtests / size_tab;
-  printf("Average = %.5f\nVariance = %.5f\n", av, var);
-  printf("Repartition for random2 (taking only values < 1 into account.\n");
-
-  for (k = 0; k < size_tab; k++)
-    {
-      chi2 += (tab[k] - th) * (tab[k] - th) / th;
-      printf("%d ", tab[k]);
-      if (((k+1) & 7) == 0)
-        printf("\n");
-    }
-
-  printf("\nChi2 statistics value (with %d degrees of freedom) : %.5f\n\n",
-         size_tab - 1, chi2);
-
-  free(tab);
-  return;
-}
-
-static void
-test_urandomb (long nbtests, mp_prec_t prec, int verbose)
-{
-  mpfr_t x;
-  int *tab, size_tab, k, sh, xn;
-  double d, av = 0, var = 0, chi2 = 0, th;
-  mp_exp_t emin;
+  mpfr_exp_t emin;
 
   size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
   tab = (int *) calloc (size_tab, sizeof(int));
@@ -250,11 +102,53 @@ test_urandomb (long nbtests, mp_prec_t prec, int verbose)
   return;
 }
 
+/* Problem reported by Carl Witty: check mpfr_urandomb give similar results
+   on 32-bit and 64-bit machines.
+   We assume the default GMP random generator does not depend on the machine
+   word size, not on the GMP version.
+*/
+static void
+bug20100914 (void)
+{
+  mpfr_t x;
+  gmp_randstate_t s;
+
+#if __MPFR_GMP(4,2,0)
+# define C1 "0.895943"
+# define C2 "0.848824"
+#else
+# define C1 "0.479652"
+# define C2 "0.648529"
+#endif
+
+  gmp_randinit_default (s);
+  gmp_randseed_ui (s, 42);
+  mpfr_init2 (x, 17);
+  mpfr_urandomb (x, s);
+  if (mpfr_cmp_str1 (x, C1) != 0)
+    {
+      printf ("Error in bug20100914, expected " C1 ", got ");
+      mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN);
+      printf ("\n");
+      exit (1);
+    }
+  mpfr_urandomb (x, s);
+  if (mpfr_cmp_str1 (x, C2) != 0)
+    {
+      printf ("Error in bug20100914, expected " C2 ", got ");
+      mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN);
+      printf ("\n");
+      exit (1);
+    }
+  mpfr_clear (x);
+  gmp_randclear (s);
+}
+
 int
 main (int argc, char *argv[])
 {
   long nbtests;
-  mp_prec_t prec;
+  mpfr_prec_t prec;
   int verbose = 0;
 
   tests_start_mpfr ();
@@ -275,17 +169,49 @@ main (int argc, char *argv[])
   else
     prec = atol(argv[2]);
 
-  test_random (nbtests, prec, verbose);
-  test_random2 (nbtests, prec, verbose);
   test_urandomb (nbtests, prec, verbose);
 
   if (argc == 1)  /* check also small precision */
     {
-      test_random (nbtests, 2, 0);
-      test_random2 (nbtests, 2, 0);
       test_urandomb (nbtests, 2, 0);
     }
 
+  bug20100914 ();
+
+#if __MPFR_GMP(4,2,0)
+  /* Get a non-zero fixed-point number whose first 32 bits are 0 with the
+     default GMP PRNG. This corresponds to the case cnt == 0 && k != 0 in
+     src/urandomb.c with the 32-bit ABI. */
+  {
+    gmp_randstate_t s;
+    mpfr_t x;
+    char *str = "0.1010111100000000000000000000000000000000E-32";
+    int k;
+
+    gmp_randinit_default (s);
+    gmp_randseed_ui (s, 4518);
+    mpfr_init2 (x, 40);
+
+    for (k = 0; k < 575123; k++)
+      {
+        mpfr_urandomb (x, s);
+        MPFR_ASSERTN (MPFR_IS_FP (x));
+      }
+
+    if (mpfr_cmp_str (x, str, 2, MPFR_RNDN) != 0)
+      {
+        printf ("Error in test_urandomb:\n");
+        printf ("Expected %s\n", str);
+        printf ("Got      ");
+        mpfr_dump (x);
+        exit (1);
+      }
+
+    mpfr_clear (x);
+    gmp_randclear (s);
+  }
+#endif
+
   tests_end_mpfr ();
   return 0;
 }
